@@ -39,7 +39,7 @@
 #include "egg/egg-secure-memory.h"
 
 #ifndef HAVE_SOCKLEN_T
-#define socklen_t int
+typedef int socklen_t;
 #endif
 
 /* The loaded PKCS#11 module */
@@ -405,7 +405,7 @@ gkd_gpg_agent_initialize (CK_FUNCTION_LIST_PTR funcs)
 
 	g_return_val_if_fail (funcs, -1);
 
-	module = gck_module_new (funcs, 0);
+	module = gck_module_new (funcs);
 	ret = gkd_gpg_agent_initialize_with_module (module);
 	g_object_unref (module);
 	return ret;
