@@ -14,7 +14,15 @@ extern void _gcr_marshal_BOOLEAN__INT (GClosure     *closure,
                                        gpointer      invocation_hint,
                                        gpointer      marshal_data);
 
-/* VOID:STRING,BOXED (gcr-marshal.list:2) */
+/* BOOLEAN:BOXED (gcr-marshal.list:2) */
+extern void _gcr_marshal_BOOLEAN__BOXED (GClosure     *closure,
+                                         GValue       *return_value,
+                                         guint         n_param_values,
+                                         const GValue *param_values,
+                                         gpointer      invocation_hint,
+                                         gpointer      marshal_data);
+
+/* VOID:STRING,BOXED (gcr-marshal.list:3) */
 extern void _gcr_marshal_VOID__STRING_BOXED (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -22,6 +30,12 @@ extern void _gcr_marshal_VOID__STRING_BOXED (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
+/* VOID:BOXED (gcr-marshal.list:4) */
+#define _gcr_marshal_VOID__BOXED	g_cclosure_marshal_VOID__BOXED
+
+/* VOID:STRING (gcr-marshal.list:5) */
+#define _gcr_marshal_VOID__STRING	g_cclosure_marshal_VOID__STRING
+
 G_END_DECLS
 
 #endif /* ___gcr_marshal_MARSHAL_H__ */
